@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with mhWaveEdit; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
 
@@ -28,9 +28,10 @@
 #include "main.h"
 #include "inifile.h"
 #include "player.h"
+#include "mainloop.h"
 
 static GtkObjectClass *parent_class;
-static guint font_height=0;
+static guint font_height=0,font_width=0;
 
 enum { VIEW_CHANGED_SIGNAL, SELECTION_CHANGED_SIGNAL, CURSOR_CHANGED_SIGNAL, 
        CHUNK_CHANGED_SIGNAL, DOUBLE_CLICK_SIGNAL, LAST_SIGNAL };
@@ -39,6 +40,9 @@ static guint chunk_view_signals[LAST_SIGNAL] = { 0 };
 static void chunk_view_update_image(ChunkView *view, guint xs, guint xe);
 static void chunk_view_redraw_samples(ChunkView *cv, off_t start, off_t end);
 static gint calc_x(ChunkView *cv, off_t sample, off_t width);
+static void chunk_view_autoscroll(gpointer timesource, 
+				  GTimeVal *current_time,
+				  gpointer user_data);
 
 static void chunk_view_changed(Document *d, gpointer user_data)
 {
@@ -353,8 +357,9 @@ static void draw_timescale(ChunkView *view, GdkEventExpose *event, gboolean text
 		    i, event->area.x+event->area.width-1, i);
      /* pixels/sec */
 
-     /* We want at least 40 pixels/major point and 8 pixels/minor point*/
-     npoints = widget->allocation.width / 40 + 1;
+     /* We want at least font_width pixels/major point 
+      * and 8 pixels/minor point*/
+     npoints = widget->allocation.width / font_width + 1;
      if (npoints < 3) npoints = 3;
      nmidpoints = npoints;
      nminorpoints = widget->allocation.width / 8 + 1;
@@ -370,14 +375,20 @@ static void draw_timescale(ChunkView *view, GdkEventExpose *event, gboolean text
 				       minorpoints, &nminorpoints,
 				       default_timescale_mode);
 			   
-
-     /* printf("nminorpoints: %d, pixels/minorpoint: %d\n",nminorpoints,
-	widget->allocation.width / nminorpoints); */
+     /*
+     printf("npoints: %d, pixels/point: %d\n",npoints,
+	    widget->allocation.width / npoints); 
+     printf("nmidpoints: %d, pixels/midpoint: %d\n",nmidpoints,
+	    (nmidpoints > 0) ? widget->allocation.width / nmidpoints : 0); 
+     printf("nminorpoints: %d, pixels/minorpoint: %d\n",nminorpoints,
+	    (nminorpoints > 0) ? widget->allocation.width / nminorpoints : 0); 
+     printf("font_width: %d\n",font_width);
+     */
 
      midtext = minortext;
 
      if (nminorpoints > 0 && 
-	 (widget->allocation.width / nminorpoints) < 40) 
+	 (widget->allocation.width / nminorpoints) < font_width) 
 	  minortext = -1;
      else
 	  midtext = -1;
@@ -487,6 +498,7 @@ static gboolean dragmode = FALSE;
 static off_t dragstart;
 static off_t dragend;
 static gboolean autoscroll = FALSE;
+static gpointer autoscroll_source = NULL;
 static ChunkView *autoscroll_view;
 static gfloat autoscroll_amount;
 static GTimeVal autoscroll_start;
@@ -622,10 +634,10 @@ static gint chunk_view_motion_notify(GtkWidget *widget, GdkEventMotion *event)
 	       if (!autoscroll) {
 		    autoscroll = TRUE;
 		    autoscroll_view = cv;
-		    g_get_current_time(&autoscroll_start);
-	       }
+		    g_get_current_time(&autoscroll_start);		    		       }
 	       if (event->x < 0.0) autoscroll_amount = event->x;
 	       else autoscroll_amount = event->x - widget->allocation.width;
+	       chunk_view_autoscroll(NULL,&autoscroll_start,NULL);
 	  } else {
 	       autoscroll = FALSE;
 	  }
@@ -652,40 +664,70 @@ static gint chunk_view_motion_notify(GtkWidget *widget, GdkEventMotion *event)
 
 #define FLOAT(x) ((gfloat)(x))
 
-gboolean chunk_view_autoscroll(void)
+static void chunk_view_autoscroll(gpointer timesource, GTimeVal *current_time,
+				  gpointer user_data)
 {
-     GTimeVal current_time,diff;
+     GTimeVal diff,new_time;
      gfloat smp;
      off_t ismp;
-     if (!autoscroll) return FALSE;
-     g_get_current_time(&current_time);
-     timeval_subtract(&diff,&current_time,&autoscroll_start);
-     if (diff.tv_sec == 0 && diff.tv_usec < 100000) return FALSE;
-     current_time.tv_usec -= current_time.tv_usec % 100000;
-     memcpy(&autoscroll_start,&current_time,sizeof(autoscroll_start));
-     smp = FLOAT(diff.tv_usec/100000 + diff.tv_sec*10) * 
-	  autoscroll_amount *
-	  FLOAT(autoscroll_view->doc->viewend-autoscroll_view->doc->viewstart)/
-	  FLOAT(GTK_WIDGET(autoscroll_view)->allocation.width);
-     if (smp < 0.0) {
-	  ismp = (off_t)(-smp);
-	  if (ismp > dragend)
-	      ismp = dragend;
-	  dragend -= ismp;
-	  document_set_selection(autoscroll_view->doc,dragend,dragstart);
-	  document_set_view(autoscroll_view->doc, dragend,
-			      autoscroll_view->doc->viewend-ismp);
-     } else {
-	  ismp = (off_t)smp;
-	  if (dragend+ismp > autoscroll_view->doc->chunk->length)
-	       ismp = autoscroll_view->doc->chunk->length - dragend;
-	  dragend += ismp;
-	  document_set_selection(autoscroll_view->doc,dragstart,dragend);
-	  document_set_view(autoscroll_view->doc,
-			    autoscroll_view->doc->viewstart+ismp,
-			    dragend);
+
+     /* puts("chunk_view_autoscroll"); */
+
+     g_assert(timesource == NULL || timesource == autoscroll_source);
+
+     if (!autoscroll) return;
+     timeval_subtract(&diff,current_time,&autoscroll_start);
+
+     /* printf("diff: %d,%d\n",diff.tv_sec,diff.tv_usec); */
+     if (diff.tv_sec > 0 || diff.tv_usec > 10000) {
+
+	  memcpy(&autoscroll_start,current_time,sizeof(autoscroll_start));
+	  
+	  /* Convert diff -> smp */
+	  smp = 0.2 * FLOAT(diff.tv_usec/10000 + diff.tv_sec*100) * 
+	       autoscroll_amount *
+	       FLOAT(autoscroll_view->doc->viewend-autoscroll_view->doc->viewstart)/
+	       FLOAT(GTK_WIDGET(autoscroll_view)->allocation.width);
+	  /* Convert smp->ismp, update view and selection */
+	  if (smp < 0.0) {
+	       ismp = (off_t)(-smp);
+	       if (ismp > dragend) {
+		    ismp = dragend;
+		    autoscroll = FALSE;
+	       }
+	       dragend -= ismp;
+	       document_set_selection(autoscroll_view->doc,dragend,dragstart);
+	       document_set_view(autoscroll_view->doc, dragend,
+				 autoscroll_view->doc->viewend-ismp);
+	  } else {
+	       ismp = (off_t)smp;
+	       if (dragend+ismp > autoscroll_view->doc->chunk->length) {
+		    ismp = autoscroll_view->doc->chunk->length - dragend;
+		    autoscroll = FALSE;
+	       }
+	       dragend += ismp;
+	       document_set_selection(autoscroll_view->doc,dragstart,dragend);
+	       document_set_view(autoscroll_view->doc,
+				 autoscroll_view->doc->viewstart+ismp,
+				 dragend);
+	  }
+     } 
+     
+     if (autoscroll) {
+	  memcpy(&new_time,current_time,sizeof(new_time));
+	  new_time.tv_usec = ((new_time.tv_usec / 25000) + 1) * 25000;
+	  if (new_time.tv_usec >= 1000000) {
+	       new_time.tv_sec += 1;
+	       new_time.tv_usec -= 1000000;
+	  }
+	  if (autoscroll_source == NULL)
+	       autoscroll_source = 
+		    mainloop_time_source_add(&new_time,chunk_view_autoscroll,
+					     NULL);
+	  else
+	       mainloop_time_source_restart(autoscroll_source,&new_time);
      }
-     return TRUE;
+
 }
 
 static gint chunk_view_button_release(GtkWidget *widget, GdkEventButton *event)
@@ -742,10 +784,13 @@ static void chunk_view_init(GtkObject *obj)
 #if GTK_MAJOR_VERSION == 1
 	  font_height = gdk_string_height( GTK_WIDGET(obj)->style->font, 
 					   "0123456789")+3;
+	  font_width = gdk_string_width( GTK_WIDGET(obj)->style->font,
+					 "0123456789")+3;
 #else
 	  PangoLayout *pl;
 	  pl = gtk_widget_create_pango_layout( GTK_WIDGET(obj), "0123456789" );
-	  pango_layout_get_pixel_size(pl, NULL, (gint *)&font_height);
+	  pango_layout_get_pixel_size(pl, (gint *)&font_width, 
+				      (gint *)&font_height);
 	  g_object_unref(pl);
 #endif
      }
