@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with mhWaveEdit; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
 
@@ -468,6 +468,7 @@ static void do_play(Mainwindow *w, off_t start, off_t end, gboolean loop)
 	  gtk_adjustment_set_value(w->speed_adj,-1.0);
      }
      document_play(w->doc, start, end, loop, -w->speed_adj->value);
+     update_desc(w);
 }
 
 
@@ -899,7 +900,7 @@ static void edit_paste(GtkMenuItem *menu_item, gpointer user_data)
 {
      Chunk *c = clipboard, *nc;
      Mainwindow *w = MAINWINDOW(user_data);
-     off_t cp;
+     off_t cp,cl;
      if (w->doc == NULL) {
 	  mainwindow_set_chunk(w,clipboard,NULL);
 	  return;
@@ -908,22 +909,22 @@ static void edit_paste(GtkMenuItem *menu_item, gpointer user_data)
      if (!dataformat_equal(&(w->doc->chunk->format),&(c->format))) {
 	  c = chunk_convert(c,&(w->doc->chunk->format),dither_editing,
 			    w->statusbar);
-	  if (c == NULL) return;
+	  if (c == NULL) return;	  
      }
-
-     nc = chunk_insert(w->doc->chunk,c,w->doc->cursorpos);
+     cl = c->length;
+     nc = chunk_insert(w->doc->chunk,c,w->doc->cursorpos);     
      gtk_object_sink(GTK_OBJECT(c));
 
      cp = w->doc->cursorpos;
-     document_update(w->doc, nc, cp, clipboard->length);
-     document_set_selection( w->doc, cp, cp + clipboard->length );
+     document_update(w->doc, nc, cp, cl);
+     document_set_selection( w->doc, cp, cp + cl );
 }
 
 static void edit_pasteover(GtkMenuItem *menuitem, gpointer user_data)
 {
      Mainwindow *w = MAINWINDOW(user_data);
      Chunk *c,*d=clipboard;
-     off_t orig_len;
+     off_t orig_len,dl;
      if (w->doc == NULL) {
 	  mainwindow_set_chunk(w,clipboard,NULL);
 	  return;
@@ -934,15 +935,16 @@ static void edit_pasteover(GtkMenuItem *menuitem, gpointer user_data)
 			    w->statusbar);
 	  if (d == NULL) return;
      }
+     dl = d->length;
 
-     orig_len = MIN(w->doc->chunk->length-w->doc->cursorpos, d->length);
+     orig_len = MIN(w->doc->chunk->length-w->doc->cursorpos, dl);
      c = chunk_replace_part(w->doc->chunk, w->doc->cursorpos, 
 			    orig_len, d);
      gtk_object_sink(GTK_OBJECT(d));
 
      document_update(w->doc, c, 0, 0);
      document_set_selection( w->doc, w->doc->cursorpos,
-			     w->doc->cursorpos + clipboard->length );
+			     w->doc->cursorpos + dl );
 }
 
 static void edit_mixpaste(GtkMenuItem *menuitem, gpointer user_data)
@@ -1485,6 +1487,11 @@ static void effects_combinechannels(GtkMenuItem *menuitem, gpointer user_data)
      mainwindow_show_effect_dialog(MAINWINDOW(user_data),"combine");
 }
 
+static void effects_sandwich(GtkMenuItem *menuitem, gpointer user_data)
+{
+     mainwindow_show_effect_dialog(MAINWINDOW(user_data),"sandwich");
+}
+
 static void view_timescale(GtkCheckMenuItem *checkmenuitem, gpointer user_data)
 {
      Mainwindow *w = MAINWINDOW ( user_data );
@@ -1887,6 +1894,7 @@ static GtkWidget *create_menu(Mainwindow *w)
 	  { N_("/Effects/Add channe_l"),NULL,effects_splitchannel,0,NULL    },
 	  { N_("/Effects/Ma_p channels..."),NULL,effects_mapchannels,0,NULL },
 	  {N_("/Effects/_Combine channels..."),NULL,effects_combinechannels,0,NULL},
+	  { N_("/Effects/Add channels from other file..."),NULL,effects_sandwich,0,NULL},
 	  { N_("/Effects/sep3"),  NULL,         NULL,           0, "<Separator>" },
 	  { N_("/Effects/_Speed adjustment..."),NULL,effects_speed, 0, NULL      },
 	  { N_("/Effects/Pipe through program..."),NULL,effects_pipe,0,NULL      },
@@ -2309,8 +2317,15 @@ static void mainwindow_state_changed(Document *d, gpointer user_data)
 static void mainwindow_value_changed(GtkAdjustment *adjustment, 
 				     gpointer user_data)
 {
-     document_set_view( MAINWINDOW(user_data)->doc, adjustment->value, 
-			adjustment->value + adjustment->page_size );
+     Document *d = MAINWINDOW(user_data)->doc;
+     off_t s,e;
+     s = (off_t)adjustment->value;
+     e = (off_t)(adjustment->value + adjustment->page_size);
+     /* Due to rounding, these can become out of range. 
+      * Especially for GTK1 where value and page_size are gfloats */
+     if (s < 0) s = 0;
+     if (e <= s) e = s+1; else if (e > d->chunk->length) e = d->chunk->length;
+     document_set_view(d, s, e);
 }
 
 static void mainwindow_zoom_changed(GtkAdjustment *adjustment, 
