@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with mhWaveEdit; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
 
@@ -202,11 +202,13 @@ static gboolean player_play_main(Chunk *chk, off_t spos, off_t epos,
      if (spos == epos) return TRUE;
      true_speed = file_speed;
      memcpy(&fmt,&(chk->format),sizeof(Dataformat));
-     i = output_select_format(&(chk->format),silent);
+     i = output_select_format(&(chk->format),silent,(GVoidFunc)player_work);
      if (i != 0) {	  
 	  if (silent) {
-	       if (!output_suggest_format(&(chk->format),&fmt))
+	       if (!output_suggest_format(&(chk->format),&fmt)) {
 		    memcpy(&fmt,&player_fallback_format,sizeof(Dataformat));
+		    fmt.channels = chk->format.channels;
+	       }
 	       
 	       b = !dataformat_samples_equal(&(chk->format),&fmt); 
 	       if (b) {
@@ -215,6 +217,12 @@ static gboolean player_play_main(Chunk *chk, off_t spos, off_t epos,
 		    gtk_object_sink(GTK_OBJECT(chk));
 		    return b;
 
+	       } else if (chk->format.channels != fmt.channels) {
+		    chk = chunk_convert_channels(chk,fmt.channels);
+		    b = player_play_main(chk,spos,epos,lp,TRUE);
+		    gtk_object_sink(GTK_OBJECT(chk));
+		    return b;
+
 	       } else if (chk->format.samplerate != fmt.samplerate) {
 
 		    if (!inifile_get_gboolean("varispeed",TRUE)) {
@@ -230,7 +238,7 @@ static gboolean player_play_main(Chunk *chk, off_t spos, off_t epos,
 			      ((float)fmt.samplerate);
 		    }
 		    /* Select the format with different sample rate */
-		    i = output_select_format(&fmt,FALSE);
+		    i = output_select_format(&fmt,FALSE,(GVoidFunc)player_work);
 		    if (i < 0) 
 			 return player_play_main(chk,spos,epos,lp,FALSE);
 		    else if (i > 0)
